import os
import sys

from discharge.exceptions import UsageError


def do_cmd():
    try:
        if len(sys.argv) < 2:
            raise UsageError()
        elif len(sys.argv) == 2 and sys.argv[1] == 'build':
            site = load_site_config()
            site.build()
        elif len(sys.argv) == 2 and sys.argv[1] == 'serve':
            site = load_site_config()
            from discharge.server import Server
            Server(site, use_reloader=True).run()
        else:
            raise UsageError()
    except UsageError:
        print "Usage:"
        print "discharge [command]"
        print "where command is one of: build, serve"
        sys.exit(1)


def load_site_config():
    settings_file = "_discharge.py"
    dct = {'__file__': os.path.realpath('_discharge.py')}
    execfile(settings_file, dct)
    return dct['site']



from __future__ import absolute_import

from markdown import Markdown
from .plugin import Plugin


class MarkdownPlugin(Plugin):
    roles = 'handler',
    markdown_file_extensions = '.mdown', '.markdown'
    markdown_extensions = 'extra', 'meta'

    def can_handle_file(self, path):
        for ext in self.markdown_file_extensions:
            if path.endswith(ext):
                return True

    def build_file(self, path, context):
        out_path = path.rsplit('.', 1)
        out_path[-1] = 'html'
        out_path = ".".join(out_path)
        page_plugin = self.site.get_plugin_by_role('page_from_content')
        with context.input_file(path) as src_file:
            with context.output_file(out_path) as dst_file:
                src = src_file.read().decode('utf-8')
                md = Markdown(
                    extensions=self.markdown_extensions,
                )
                content = md.convert(src)
                meta = {'title': ''}
                meta.update(md.Meta)
                html = page_plugin.page_from_content(
                    out_path,
                    content,
                    **meta
                )
                dst_file.write(html.encode('utf-8'))



import os
import shutil

from .exceptions import DuplicateHandlers
from .buildcontext import BuildContext


class Site(object):

    def __init__(self, source_path, build_path=None, base_path=''):
        if build_path is None:
            build_path = os.path.join(source_path, '_build')

        self.source_path = source_path
        self.build_path = build_path
        self.base_path = base_path
        self.plugins = []
        self.plugins_by_role = {}

    def add_plugin(self, plugin):
        if plugin in self.plugins:
            raise ValueError("Plugin already registered")
        self.plugins.append(plugin)
        plugin.add_to_site(self)

    def register_role(self, plugin, role):
        role_plugins = self.plugins_by_role.get(role, [])
        if plugin in role_plugins:
            raise ValueError("Plugin already registered for role %s" % role)
        role_plugins.append(plugin)
        self.plugins_by_role[role] = role_plugins

    def get_plugin_by_role(self, role):
        plugins = self.plugins_by_role.get(role, [])
        if len(plugins) == 0:
            raise ValueError("No plugin for role %s" % role)
        elif len(plugins) > 1:
            raise ValueError("Multiple plugins for role %s" % role)
        else:
            return list(plugins)[0]

    def get_plugins_by_role(self, role):
        return list(self.plugins_by_role.get(role, []))

    def clean(self):
        if os.path.exists(self.build_path):
            shutil.rmtree(self.build_path)

    def build(self):
        assert os.path.isdir(self.source_path), \
            "Site not found at %s" % self.source_path

        self.clean()

        context = BuildContext(self)

        walker = os.walk(self.source_path)
        for dirpath, dirnames, filenames in walker:
            dirpath = dirpath.split(self.source_path, 1)[1]
            dirpath = dirpath.strip('/\\')

            hidden_dirnames = list(
                name for name in dirnames
                if name.startswith('_') or name.startswith('.')
            )
            hidden_filenames = list(
                name for name in filenames
                if name.startswith('_') or name.startswith('.')
            )

            for name in hidden_dirnames:
                dirnames.remove(name)

            for name in hidden_filenames:
                filenames.remove(name)

            os.mkdir(os.path.join(self.build_path, dirpath))

            for filename in filenames:
                file_path = os.path.join(dirpath, filename)

                handlers = self.get_plugins_by_role('handler')
                handlers = [plugin for plugin in handlers
                            if plugin.can_handle_file(file_path)]

                if len(handlers) > 1:
                    raise DuplicateHandlers(
                        "Multiple handlers for file '%s': %s" % (
                            file_path,
                            repr(handlers)))
                elif len(handlers) == 1:
                    handlers[0].build_file(file_path, context)
                else:
                    with context.input_file(file_path) as src:
                        with context.output_file(file_path) as dst:
                            shutil.copyfileobj(src, dst)

        for plugin in self.get_plugins_by_role('producer'):
            plugin.produce(context)

        return context



class Plugin(object):
    roles = ()

    def __init__(self):
        site = None

    def add_to_site(self, site):
        self.site = site
        for role in self.roles:
            self.register_role(role)

    def register_role(self, role):
        self.site.register_role(self, role)



from .plugin import Plugin
from ..exceptions import FileExists


class RobotsPlugin(Plugin):
    roles = 'producer',

    def produce(self, context):
        try:
            with context.output_file('robots.txt', 'wb') as f:
                f.write("User-agent: *".encode("utf-8"))
        except FileExists:
            pass



import os
import time
from urllib2 import urlopen, HTTPError
from threading import Thread
from werkzeug.wrappers import Request
from werkzeug.exceptions import NotFound
from werkzeug.serving import run_simple, WSGIRequestHandler
from werkzeug.wsgi import get_path_info, wrap_file
import mimetypes


class CustomRequestHandler(WSGIRequestHandler):
    """Fixes Werkzeug's WSGIRequestHandler, which doesn't shut down
    properly on python 2.6
    """

    def initiate_shutdown(self):
        if os.environ.get('WERKZEUG_RUN_MAIN') == 'true':
            super(CustomRequestHandler, self).initiate_shutdown()
        else:
            self.server.shutdown()
            self.server.socket.close()


class StaticFilesMiddleWare():
    def __init__(self, application, site):
        self.application = application
        self.site = site
        self.context = None
        self.extra_files = []

    def __call__(self, environ, start_response):
        if self.context is None:
            self.context = self.site.build()

            for input_file in self.context.input_files:
                self.extra_files.append(
                    os.path.join(self.site.source_path, input_file))

        cleaned_path = get_path_info(environ)
        cleaned_path = cleaned_path.lstrip('/')
        cleaned_path = '/' + cleaned_path
        static_file_path = None
        if cleaned_path.lstrip('/') in self.context.output_files:
            static_file_path = cleaned_path
        elif cleaned_path.endswith('/'):
            try_cleaned_path = cleaned_path + 'index.html'
            if try_cleaned_path.lstrip('/') in self.context.output_files:
                static_file_path = try_cleaned_path

        if static_file_path is None:
            return self.application(environ, start_response)

        real_path = os.path.join(self.context.site.build_path,
                                 static_file_path.lstrip('/'))

        guessed_type = mimetypes.guess_type(real_path)
        mime_type = guessed_type[0] or 'text/plain'
        file_size = int(os.path.getsize(real_path))

        headers = [
            ('Content-Type', mime_type),
            ('Content-Length', str(file_size)),
        ]
        start_response('200 OK', headers)
        return wrap_file(environ, open(real_path, 'rb'))


class Server(Thread):

    def __init__(self, site, host='localhost', port=8000, use_reloader=False):
        super(Server, self).__init__()
        self.site = site
        self.host = host
        self.port = port
        self.use_reloader = use_reloader

    @Request.application
    def application(self, request):
        if request.path == '/!SHUTDOWN!':
            request.environ['werkzeug.server.shutdown']()
        return NotFound()

    def run(self):
        application = StaticFilesMiddleWare(self.application, self.site)
        extra_files = application.extra_files

        run_simple(
            self.host, self.port, application,
            request_handler=CustomRequestHandler,
            threaded=True,
            use_reloader=self.use_reloader,
            extra_files=extra_files,
            use_debugger=True,
        )

    def start(self, *args, **kwargs):
        ret = super(Server, self).start()
        wait = kwargs.pop('wait', False)
        if wait:
            self.wait_for_start()
        return ret

    def shutdown(self, wait=False):
        if self.is_alive():
            try:
                urlopen('http://%s:%s/!SHUTDOWN!' % (self.host, self.port))
            except HTTPError:
                pass
        if wait:
            self.wait_for_shutdown()
        self.join()

    def wait_for_start(self):
        tries = 500
        print "waiting..."
        while tries:
            try:
                urlopen('http://%s:%s/!PING!' % (self.host, self.port))
            except HTTPError:
                break
            except:
                pass
            tries -= 1
            time.sleep(0.01)

        if  not tries:
            raise Exception("Could not start server")

    def wait_for_shutdown(self):
        print "waiting..."
        tries = 500
        while tries:
            try:
                urlopen('http://%s:%s/!PING!' % (self.host, self.port))
            except HTTPError:
                pass
            except:
                break
            tries -= 1
            time.sleep(0.01)

        if not tries:
            raise Exception("Could not stop server")



from setuptools import setup, find_packages
import discharge

setup(
    name='Discharge',
    version=discharge.version,
    author='Richard Ward',
    author_email='richard@richard.ward.name',
    url='https://github.com/RichardOfWard/discharge',
    description='A static site generator',
    long_description=open('README.rst').read(),
    packages=find_packages(exclude=['tests']),
    scripts=['scripts/discharge'],
    install_requires=[
        'Jinja2',
        'jinja2-highlight',
        'Markdown',
        'Werkzeug',
    ],
    tests_require=['nose'],
    test_suite='nose.collector',
)



import os

from .exceptions import FileExists


class BuildContext(object):
    def __init__(self, site):
        self.site = site
        self.input_files = set()
        self.output_files = set()

    def input_file(self, path, mode='rb', buffering=-1):
        full_path = os.path.join(self.site.source_path, path)
        f = open(full_path, mode, buffering)
        self.input_files.add(path)
        return f

    def output_file(self, path, mode='wb', buffering=-1):
        full_path = os.path.join(self.site.build_path, path)
        if os.path.exists(full_path):
            raise FileExists("File %s already exists" % full_path)
        f = open(full_path, mode, buffering)
        self.output_files.add(path)
        return f



import jinja2

from .plugin import Plugin


class Jinja2TemplatesPlugin(Plugin):
    roles = 'handler', 'page_from_content',

    def can_handle_file(self, path):
        return path.endswith('.html')

    def add_to_site(self, site):
        super(Jinja2TemplatesPlugin, self).add_to_site(site)
        self.loader = jinja2.FileSystemLoader(self.site.source_path)
        self.env = jinja2.Environment(
            loader=self.loader,
            autoescape=True,
            undefined=jinja2.StrictUndefined,
            extensions=[
                'jinja2.ext.autoescape',
                'jinja2_highlight.HighlightExtension',
            ],
        )

    def build_file(self, path, context):
        with context.input_file(path):
            with context.output_file(path) as f:
                f.write(self.render_template(path, path).encode('utf-8'))

    def render_template(self, template_name, path, **kwargs):
        context = {
            'file_path': '/' + path.lstrip('/'),
            'base_path': self.site.base_path,
        }
        context.update(kwargs)
        template = self.env.get_template(template_name)
        return template.render(**context)

    def page_from_content(self, path, content, **kwargs):
        return self.render_template(
            '_page.html',
            path,
            content=content,
            **kwargs
        )
